#include "media-input.h"

#include <boost/tuple/tuple.hpp>

#ifndef __DYNAMIC_INPUT_DEVICE_PROVIDER_H__
#define __DYNAMIC_INPUT_DEVICE_PROVIDER_H__

#define DYNAMIC_INPUT_DEVICE_PROVIDER_NAME "dynamic-input-device-provider"

namespace Ekiga {

  /* This code add devices which will only available sometimes in
   * ekiga (FIXME: should use gstreamer's new device-detection
   * framework when available)
   */
  class DynamicInputDeviceProvider:
    public InputDevice::Provider
  {
  public:
    DynamicInputDeviceProvider ();
    ~DynamicInputDeviceProvider ();

    const std::string get_name () const
    { return DYNAMIC_INPUT_DEVICE_PROVIDER_NAME; }

    void add_input_device (Ekiga::MediaSessionPtr session,
			   InputDevice::DescriptorPtr descriptor);

    void remove_input_device (Ekiga::MediaSessionPtr session,
			      InputDevice::DescriptorPtr descriptor);

    bool visit_devices (boost::function1<bool, InputDevice::DescriptorPtr> visitor) const;

  private:
    std::vector<InputDevice::DescriptorPtr> descriptors;

    typedef std::list<boost::tuple<boost::weak_ptr<Ekiga::MediaSession>, InputDevice::DescriptorPtr, GstElement*> > devices_type;
    devices_type devices;

    void cleanup_dead_sessions_devices ();
  };

};

#endif
