#include "gtk-combo-input-device.h"

enum {
  COLUMN_DISPLAY_NAME,
  COLUMN_PROVIDER,
  COLUMN_DESCRIPTION,
};

struct combo_data
{
  Ekiga::InputDevice::Type device_type;
  boost::shared_ptr<Ekiga::MediaInput> input;
  GtkListStore* store;
};

static void
combo_data_destroyer (combo_data* data)
{
  delete data;
}

static bool
find_iter_for_descriptor (GtkTreeModel* model,
			  Ekiga::InputDevice::DescriptorPtr descriptor,
			  GtkTreeIter* iter)
{
  bool result = false;

  if (gtk_tree_model_get_iter_first (model, iter)) {

    bool go_on = true;
    while (go_on) {

      gchar* provider = NULL;
      gchar* description = NULL;
      gtk_tree_model_get (model, iter,
			  COLUMN_PROVIDER, &provider,
			  COLUMN_DESCRIPTION, &description,
			  -1);

      if (descriptor->provider == provider and descriptor->description == description) {

	result = true;
	go_on = false;
      }
    g_free (provider);
    g_free (description);

      if (go_on)
	go_on = (gtk_tree_model_iter_next (model, iter) == TRUE);
    }
  }

  return result;
}

static void
input_device_added (Ekiga::InputDevice::DescriptorPtr descriptor,
		    combo_data* data)
{
  // we don't care about anything else!
  if (descriptor->device_type != data->device_type)
    return;

  GtkTreeIter iter;
  gtk_list_store_append (data->store, &iter);
  gtk_list_store_set (data->store, &iter,
		      COLUMN_DISPLAY_NAME, descriptor->display_name.c_str (),
		      COLUMN_PROVIDER, descriptor->provider.c_str (),
		      COLUMN_DESCRIPTION, descriptor->description.c_str (),
		      -1);
}

static void
input_device_removed (Ekiga::InputDevice::DescriptorPtr descriptor,
		      combo_data* data)
{
  // we don't care about anything else!
  if (descriptor->device_type != data->device_type)
    return;

  GtkTreeIter iter;
  if (find_iter_for_descriptor (GTK_TREE_MODEL (data->store), descriptor, &iter)) {

	gtk_list_store_remove (data->store, &iter);
  }
}

static bool
input_devices_visitor (Ekiga::InputDevice::DescriptorPtr descriptor,
		       combo_data* data)
{
  input_device_added (descriptor, data);
  return true;
}

static void
change_input_device (GtkComboBox* box,
		    combo_data* data)
{
  GtkTreeIter iter;

  if (gtk_combo_box_get_active_iter (box, &iter)) {

    gchar* display_name = NULL;
    gchar* provider = NULL;
    gchar* description = NULL;
    gtk_tree_model_get (GTK_TREE_MODEL (data->store), &iter,
			COLUMN_DISPLAY_NAME, &display_name,
			COLUMN_PROVIDER, &provider,
			COLUMN_DESCRIPTION, &description,
			-1);
    Ekiga::InputDevice::DescriptorPtr descriptor(new Ekiga::InputDevice::Descriptor);
    descriptor->display_name = display_name;
    descriptor->provider = provider;
    descriptor->description = description;
    descriptor->device_type = data->device_type;
    data->input->set_default_device (descriptor);
    g_free (display_name);
    g_free (provider);
    g_free (description);
  }
}

GtkWidget*
gtk_combo_input_device_new (boost::shared_ptr<Ekiga::MediaInput> input,
			    Ekiga::InputDevice::Type device_type)
{
  combo_data* data = new combo_data;
  data->device_type = device_type;
  data->input = input;
  data->store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
  GtkWidget* combo = gtk_combo_box_new_with_model (GTK_TREE_MODEL (data->store));
  g_object_set_data_full (G_OBJECT (combo), "combo_data", data,
			  (GDestroyNotify)combo_data_destroyer);
  GtkCellRenderer* renderer = gtk_cell_renderer_text_new ();
  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, TRUE);
  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo), renderer,
				  "text", COLUMN_DISPLAY_NAME,
				  NULL);
  data->input->visit_devices (boost::bind (&input_devices_visitor, _1, data));
  Ekiga::InputDevice::DescriptorPtr descriptor
    = data->input->get_default_device (device_type);
  GtkTreeIter iter;
  if (find_iter_for_descriptor (GTK_TREE_MODEL (data->store), descriptor, &iter)) {

    gtk_combo_box_set_active_iter (GTK_COMBO_BOX (combo), &iter);
  }

  g_signal_connect (combo, "changed", (GCallback)change_input_device, data);
  data->input->device_added.connect (boost::bind(&input_device_added, _1, data));
  data->input->device_removed.connect (boost::bind (&input_device_removed, _1, data));

  return combo;
}
