#include <list>
#include <map>

#include <gst/gst.h>
#include <boost/smart_ptr.hpp>

#ifndef __MEDIA_SESSION_H__
#define __MEDIA_SESSION_H__

namespace Ekiga {

  /* deleter for use in boost shared pointers */
  struct gst_element_deleter
  {
    void operator()(void const* elt) const
    {
      gst_element_set_state ((GstElement*)elt, GST_STATE_NULL);
      gst_object_unref ((gpointer)elt);
    }
  };

  class MediaSession
  {
  public:

    MediaSession ();

    ~MediaSession ();

    /*
     * This part of the api is about the main pipeline
     */

  public:

    bool add_input_device (GstElement* device);

    bool add_output_device (GstElement* device);

    bool remove_input_device (GstElement* device);

    void start_media_pipeline ();

    void stop_media_pipeline ();

  public: // but should be private and visible through a populate_menu!

    void start_recording ();

    void stop_recording ();

  private:
    GstElement* pipeline;
    GstElement* recorder;
    bool is_running;
    bool is_recording;

    /*
     * This part of the api is for audio events
     */
  public:

    void play_file_once (const std::string filename);

    void play_file_repeat (const std::string filename);

    void stop_file_repeat (const std::string filename);

  private:
    std::map<std::string, boost::shared_ptr<GstElement> > repeating_play_files;
  };
  typedef boost::shared_ptr<MediaSession> MediaSessionPtr;
}

#endif
