#include "static-input-device-provider.h"

Ekiga::StaticInputDeviceProvider::StaticInputDeviceProvider()
{
  InputDevice::DescriptorPtr descriptor;

  descriptor = InputDevice::DescriptorPtr(new InputDevice::Descriptor);
  descriptor->display_name = "Audio test input"; // FIXME: translate
  descriptor->provider = get_name ();
  descriptor->description = "audiotestsrc name=local_audio_hook";
  descriptor->device_type = InputDevice::Audio;
  descriptors.push_back (descriptor);

  descriptor = InputDevice::DescriptorPtr(new InputDevice::Descriptor);
  descriptor->display_name = "Video test input"; // FIXME: translate
  descriptor->provider = get_name ();
  descriptor->description = "videotestsrc ! videoconvert ! video/x-raw,format=I420,width=320,height=240 ! videomixer name=local_video_hook";
  descriptor->device_type = InputDevice::Video;
  descriptors.push_back (descriptor);

  // FIXME: that one is just here so we can test for now
  descriptor = InputDevice::DescriptorPtr(new InputDevice::Descriptor);
  descriptor->display_name = "Goom video test input";
  descriptor->provider = get_name ();
  descriptor->description = "audiotestsrc ! goom ! videoconvert ! video/x-raw,format=I420,width=320,height=240 ! videomixer name=local_video_hook";
  descriptor->device_type = InputDevice::Video;
  descriptors.push_back (descriptor);
}

Ekiga::StaticInputDeviceProvider::~StaticInputDeviceProvider()
{
}

void
Ekiga::StaticInputDeviceProvider::add_input_device (Ekiga::MediaSessionPtr session,
						    InputDevice::DescriptorPtr descriptor)
{
  GstElement* element = gst_parse_bin_from_description (descriptor->description.c_str (),
							FALSE, NULL);
  gst_element_set_name (element, descriptor->display_name.c_str ());
  session->add_input_device (element);
  devices.push_front (boost::make_tuple(boost::weak_ptr<Ekiga::MediaSession>(session),
					descriptor, element));

  // should be reasonable
  if (devices.size () > 0 && devices.size () % 10 == 0)
    cleanup_dead_sessions_devices ();
}

void
Ekiga::StaticInputDeviceProvider::remove_input_device (Ekiga::MediaSessionPtr session,
						       InputDevice::DescriptorPtr descriptor)
{
  devices_type::iterator iter = devices.begin ();

  for (; iter != devices.end (); ++iter) {

    Ekiga::MediaSessionPtr other_session = (*iter).get<0> ().lock ();
    InputDevice::DescriptorPtr other_descriptor = (*iter).get<1> ();
    /* IMPORTANT: the gui may have rebuilt the descriptor to match
     * this data, so we have to actually check for it instead of going
     * the easy way of other_descriptor == descriptor!
     */
    if (other_session == session
	&& other_descriptor->provider == descriptor->provider
	&& other_descriptor->description == descriptor->description
	&& other_descriptor->device_type == descriptor->device_type)
      break;
  }
  if (iter != devices.end ()) {

    session->remove_input_device ((*iter).get<2> ());
    devices.erase (iter);
  }
}

bool
Ekiga::StaticInputDeviceProvider::visit_devices (boost::function1<bool, InputDevice::DescriptorPtr> visitor) const
{
  bool go_on = true;
  for (std::vector<InputDevice::DescriptorPtr>::const_iterator iter = descriptors.begin ();
       iter != descriptors.end () and go_on;
       ++iter) {

    go_on = visitor (*iter);
  }
  return go_on;
}

void
Ekiga::StaticInputDeviceProvider::cleanup_dead_sessions_devices ()
{
  devices_type::iterator iter = devices.begin ();
  while (iter != devices.end ()) {

    Ekiga::MediaSessionPtr session = (*iter).get<0> ().lock ();

    if (!session) // He's dead, Jim!
      devices.erase (iter);
    else
      ++iter;
  }
}
