#include "media-input.h"

#include <boost/tuple/tuple.hpp>

#ifndef __STATIC_INPUT_DEVICE_PROVIDER_H__
#define __STATIC_INPUT_DEVICE_PROVIDER_H__

namespace Ekiga {

  /* This code add devices which ekiga will always make
   * available: audio and video test, display...
   */
  class StaticInputDeviceProvider:
    public InputDevice::Provider
  {
  public:
    StaticInputDeviceProvider ();
    ~StaticInputDeviceProvider ();

    const std::string get_name () const
    { return "static-input-device-provider"; }

    void add_input_device (Ekiga::MediaSessionPtr session,
			   InputDevice::DescriptorPtr descriptor);

    void remove_input_device (Ekiga::MediaSessionPtr session,
			      InputDevice::DescriptorPtr descriptor);

    bool visit_devices (boost::function1<bool, InputDevice::DescriptorPtr> visitor) const;

    /* This is made public because the main media input class will
     * want a direct access to some of those:
     * - index 0 MUST be audio
     * - index 1 MUST be video
     */
    std::vector<InputDevice::DescriptorPtr> descriptors;

  private:
    typedef std::list<boost::tuple<boost::weak_ptr<Ekiga::MediaSession>, InputDevice::DescriptorPtr, GstElement*> > devices_type;
    devices_type devices;

    void cleanup_dead_sessions_devices ();
  };

};

#endif
